﻿#include "xx_rgb_pwm.h"
#include <stdio.h>
#include <string.h>
#include "app_util_platform.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "app_scheduler.h"


#define m_demo1_top 255
#define COLOR_DISPLAY_NUM  3

nrf_drv_pwm_t m_pwm0 = NRF_DRV_PWM_INSTANCE(0);
color_t color[]={ {255, 0, 255}, {255, 255, 0},{0, 255, 255}, {0, 0, 0}, {255, 0, 0}, {0, 0, 255}, {0, 255, 0}};

static nrf_pwm_values_individual_t p_rgb_buffer1[255];
static nrf_pwm_values_individual_t p_rgb_buffer2[255];
static  int index = 0;
static  int repeat = 10; 

bool finish_flag = true ;       /**< 首次更新pwm填充用buffer2. */


static nrf_pwm_sequence_t   const  m_demo1_seq =
{
    .values.p_individual = p_rgb_buffer1,
    .length              = NRF_PWM_VALUES_LENGTH(p_rgb_buffer1),
    .repeats             = 103,
    .end_delay           = 0
};



void rgb_gradual_handler(void * p_event_data, uint16_t event_size)
{
    NRF_LOG_RAW_INFO("into rgb_handler!!!!!!!!!!!! \r\n");
    static uint8_t index = 1;
    uint8_t buffer_num = *(uint8_t *)p_event_data;
    if (index > (COLOR_DISPLAY_NUM -2)){                                              /**< 队列循环尾部到头部的渐变. */
        if (buffer_num ==2) {
            rgb_gradual_change_one_to_one(p_rgb_buffer2, 0, color[index], color[0]);
            // NRF_LOG_RAW_INFO("p_rgb_buffer2 [%d:%d]\r\n", index, 0);
        } else {
            rgb_gradual_change_one_to_one(p_rgb_buffer1, 0, color[index], color[0]);
            // NRF_LOG_RAW_INFO("p_rgb_buffer1 [%d:%d]\r\n", index, 0);
        }
        index = 0;
    } else {                                                                          /**< 中间过程的渐变. */
        if(buffer_num ==2) {
            rgb_gradual_change_one_to_one(p_rgb_buffer2, 0, color[index], color[index+1]);
        } else {
            rgb_gradual_change_one_to_one(p_rgb_buffer1, 0, color[index], color[index+1]);
        }
        index ++;
    }
    NRF_LOG_RAW_INFO("out rgb_handler @@@@@@@@@@@@@@ \r\n\r\n");
}


static void demo1_handler(nrf_drv_pwm_evt_type_t event_type)
{
    uint8_t buffer_num = 0;
    nrf_pwm_values_t values;
    if (event_type == NRF_DRV_PWM_EVT_FINISHED)
    {
        if(finish_flag) {
            finish_flag =false;

            /**< 填充buffer2给pwm. */
            values.p_individual= p_rgb_buffer2;
            nrf_drv_pwm_sequence_values_update(&m_pwm0,  1,  values);
            NRF_LOG_RAW_INFO("play buffer2  buf buffer1\r\n");

            /**< 更新buffer1. */
            buffer_num = 1;
            app_sched_event_put(&buffer_num, sizeof(buffer_num), rgb_gradual_handler);

        } else {
            finish_flag = true;

            /**< 填充buffer1给pwm. */
            values.p_individual= p_rgb_buffer1;
            nrf_drv_pwm_sequence_values_update(&m_pwm0,  1,  values); 
            NRF_LOG_RAW_INFO("play buffer1 buf buffer2\r\n");

            /**< 更新buffer2. */      
            buffer_num = 2;
            app_sched_event_put(&buffer_num, sizeof(buffer_num), rgb_gradual_handler);                                                                                                                                                                                                              
        }
    }
}

 
 void rgb_pwm_start()
{
    rgb_gradual_change_one_to_one(p_rgb_buffer1, 0, color[0] , color[0]);   /**<初始化buffer1. */

    (void)nrf_drv_pwm_simple_playback(&m_pwm0, &m_demo1_seq, 1,             /**<播放buffer1. */
                                      NRF_DRV_PWM_FLAG_LOOP);

    rgb_gradual_change_one_to_one(p_rgb_buffer2, 0, color[0] , color[1]);   /**<初始化buffer2. */
}


void rgb_pwm_init(void)
{
          
    nrf_drv_pwm_config_t const config0 =
    {
        .output_pins =
        {
            RGB_R_Pin | NRF_DRV_PWM_PIN_INVERTED, // channel 0
            RGB_G_Pin | NRF_DRV_PWM_PIN_INVERTED, // channel 1
            RGB_B_Pin | NRF_DRV_PWM_PIN_INVERTED, // channel 2
            NRF_DRV_PWM_PIN_NOT_USED  // channel 3
        },
        .irq_priority = APP_IRQ_PRIORITY_LOWEST,
        .base_clock   = NRF_PWM_CLK_2MHz,
        .count_mode   = NRF_PWM_MODE_UP,
        .top_value    = m_demo1_top,
        .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
        .step_mode    = NRF_PWM_STEP_AUTO
    };

      
    APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm0, &config0, demo1_handler));
  
}

